@@ -2,7 +2,7 @@
 set_project("xmake")
 
 -- version
-set_version("2.9.5", {build = "%Y%m%d"})
+set_version("2.9.6", {build = "%Y%m%d"})
 
 -- set xmake min version
 set_xmakever("2.8.5")
@@ -131,7 +131,7 @@ if is_plat("windows") then
 end
 
 -- add projects
-includes("src/sv", "src/lz4", "src/tbox", "src/xmake", "src/demo")
+includes("src/sv", "src/lz4", "src/tbox", "src/xmake", "src/cli")
 if has_config("lua_cjson") then
     includes("src/lua-cjson")
 end
