@@ -21,6 +21,7 @@
 -- imports
 import("core.base.option")
 import("core.base.semver")
+import("core.base.hashset")
 import("core.tool.toolchain")
 import("core.project.config")
 import("core.project.project")
@@ -119,7 +120,7 @@ end
 function _get_cflags_from_packagedeps(package, opt)
     local values
     for _, depname in ipairs(opt.packagedeps) do
-        local dep = type(depname) ~= "string" and depname or package:dep(depname)
+        local dep = type(depname) ~= "string" and depname or package:librarydep(depname)
         if dep then
             local fetchinfo = dep:fetch()
             if fetchinfo then
@@ -151,7 +152,7 @@ end
 function _get_ldflags_from_packagedeps(package, opt)
     local values
     for _, depname in ipairs(opt.packagedeps) do
-        local dep = type(depname) ~= "string" and depname or package:dep(depname)
+        local dep = type(depname) ~= "string" and depname or package:librarydep(depname)
         if dep then
             local fetchinfo = dep:fetch()
             if fetchinfo then
@@ -796,7 +797,7 @@ function _get_default_flags(package, configs, buildtype, opt)
     local cachekey = buildtype .. package:plat() .. package:arch()
     local cmake_default_flags = _g.cmake_default_flags and _g.cmake_default_flags[cachekey]
     if not cmake_default_flags then
-        local tmpdir = path.join(os.tmpdir() .. ".dir", package:name(), package:mode())
+        local tmpdir = path.join(os.tmpfile() .. ".dir", package:displayname(), package:mode())
         local dummy_cmakelist = path.join(tmpdir, "CMakeLists.txt")
 
         io.writefile(dummy_cmakelist, format([[
@@ -1006,7 +1007,7 @@ end
 function _build_for_msvc(package, configs, opt)
     local allbuild = os.isfile("ALL_BUILD.vcxproj") and "ALL_BUILD.vcxproj" or "ALL_BUILD.vcproj"
     assert(os.isfile(allbuild), "ALL_BUILD project not found!")
-    msbuild.build(package, {allbuild, "-t:Rebuild"})
+    msbuild.build(package, {allbuild, "-t:Rebuild"}, opt)
 end
 
 -- do build for make
@@ -1079,10 +1080,10 @@ end
 function _install_for_msvc(package, configs, opt)
     local allbuild = os.isfile("ALL_BUILD.vcxproj") and "ALL_BUILD.vcxproj" or "ALL_BUILD.vcproj"
     assert(os.isfile(allbuild), "ALL_BUILD project not found!")
-    msbuild.build(package, {allbuild, "-t:Rebuild", "/nr:false"})
+    msbuild.build(package, {allbuild, "-t:Rebuild", "/nr:false"}, opt)
     local projfile = os.isfile("INSTALL.vcxproj") and "INSTALL.vcxproj" or "INSTALL.vcproj"
     if os.isfile(projfile) then
-        msbuild.build(package, {projfile})
+        msbuild.build(package, {projfile}, opt)
         os.trycp("install/bin", package:installdir())
         os.trycp("install/lib", package:installdir()) -- perhaps only headers library
         os.trycp("install/share", package:installdir())
@@ -1157,7 +1158,11 @@ function _get_cmake_generator(package, opt)
     opt = opt or {}
     local cmake_generator = opt.cmake_generator
     if not cmake_generator then
-        if project.policy("package.cmake_generator.ninja") or package:policy("package.cmake_generator.ninja") then
+        local use_ninja = package:policy("package.cmake_generator.ninja")
+        if use_ninja == nil then
+            use_ninja = project.policy("package.cmake_generator.ninja")
+        end
+        if use_ninja then
             cmake_generator = "Ninja"
         end
         if not cmake_generator then
@@ -1197,6 +1202,38 @@ function configure(package, configs, opt)
             table.insert(argv, "-D" .. name .. "=" .. value)
         end
     end
+    -- shrink cmake arguments, fix too long arguments
+    -- @see https://github.com/xmake-io/xmake-repo/pull/5247#discussion_r1780302212
+    local cmake_argv = {}
+    local long_options = hashset.of(
+        "CMAKE_C_FLAGS",
+        "CMAKE_CXX_FLAGS",
+        "CMAKE_ASM_FLAGS",
+        "CMAKE_EXE_LINKER_FLAGS",
+        "CMAKE_SHARED_LINKER_FLAGS",
+        "CMAKE_C_FLAGS_RELEASE",
+        "CMAKE_CXX_FLAGS_RELEASE",
+        "CMAKE_ASM_FLAGS_RELEASE",
+        "CMAKE_EXE_LINKER_FLAGS_RELEASE",
+        "CMAKE_SHARED_LINKER_FLAGS_RELEASE",
+        "CMAKE_C_FLAGS_DEBUG",
+        "CMAKE_CXX_FLAGS_DEBUG",
+        "CMAKE_ASM_FLAGS_DEBUG",
+        "CMAKE_EXE_LINKER_FLAGS_DEBUG",
+        "CMAKE_SHARED_LINKER_FLAGS_DEBUG")
+    local shrink = false
+    table.remove_if(argv, function (idx, value)
+        local k, v = value:match("%-D(.*)=(.*)")
+        if k and v and long_options:has(k) and #v > 128 then
+            table.insert(cmake_argv, ("set(%s \"%s\")"):format(k, tostring(v)))
+            shrink = true
+            return true
+        end
+    end)
+    if shrink then
+        local cmakefile = path.join(opt.curdir and opt.curdir or oldir, "CMakeLists.txt")
+        io.insert(cmakefile, 1, table.concat(cmake_argv, "\n"))
+    end
     table.insert(argv, oldir)
 
     -- do configure
